from django.utils.translation import gettext_lazy as _

from collections import OrderedDict
import datetime

from django.templatetags.static import static
from django.utils.timezone import now

try:
    from debug_toolbar.panels import Panel
except ImportError:
    # django-debug-toolbar 1.x back compatibility
    from debug_toolbar.panels import DebugPanel as Panel

from .conf import MAIL_TOOLBAR_TTL
from .utils import load_outbox, save_outbox
from .urls import urlpatterns


class MailToolbarPanel(Panel):
    """
    Panel that displays informations about mail
    """
    name = 'Mail'
    template = 'mail_panel/panel.html'
    has_content = True
    mail_list = OrderedDict()

    @property
    def scripts(self):
        scripts = super().scripts
        scripts.append(static("debug_toolbar/mail/toolbar.mail.js"))
        return scripts

    def nav_title(self):
        return _('Mail')

    def nav_subtitle(self):
        mail_list = load_outbox()
        unread_count = 0
        for message in list(mail_list.values()):
            if not message.read:
                unread_count += 1
        if unread_count == 1:
            return '{0} unread message'.format(unread_count)
        elif unread_count > 1:
            return '{0} unread messages'.format(unread_count)
        return ''

    def title(self):
        return _('Mail')

    def generate_stats(self, request, response):
        """
        Main panel view.  Loads and displays listing of mail.
        """

        mail_list = load_outbox()
        if mail_list == {}:
            return
        mail_list = OrderedDict(
            sorted(iter(mail_list.items()),
            key=lambda x: x[1].date_sent,
            reverse=True
        ))
        prev_len = len(mail_list)

        # Expire messages past TTL
        expire_at = now() - datetime.timedelta(
            seconds=MAIL_TOOLBAR_TTL)
        for message_id, message in list(mail_list.items()):
            if message.date_sent < expire_at:
                del mail_list[message_id]

        if prev_len != len(mail_list):
            save_outbox(mail_list)

        self.mail_list = mail_list
        self.record_stats({
            'mail_list': self.mail_list,
        })

    def process_response(self, request, response):
        """
        generate_stats replace process_response in django-debug-toolbar 2.0.
        Call generate_stats for back compatibility.
        """
        self.generate_stats(request, response)

    @classmethod
    def get_urls(cls):
        return urlpatterns



from django.core import mail
from django.core.mail.backends.locmem import EmailBackend
from django.utils.timezone import now
import datetime
from uuid import uuid4

from .utils import load_outbox, save_outbox


class MailToolbarBackendEmail(mail.EmailMultiAlternatives):
    def __init__(self, message):
        try:
            self.id = uuid4().get_hex()
        except AttributeError:
            self.id = uuid4().hex  # python 3
        self.date_sent = now()
        self.read = False
        message.message()  # triggers header validation

        super(MailToolbarBackendEmail, self).__init__(
            subject=message.subject,
            to=message.to,
            cc=message.cc,
            bcc=message.bcc,
            reply_to=message.reply_to,
            from_email=message.from_email,
            body=message.body,
            alternatives=message.alternatives,
            headers=message.extra_headers,
        )


class MailToolbarBackend(EmailBackend):
    """A email backend for use during testing.

    The test connection stores email messages in a dummy outbox,
    rather than sending them out on the wire.  This is cached and
    can be accessed via the memmail interface.

    Also store in LocMem for compatibility with tests.
    """

    def __init__(self, *args, **kwargs):
        super(MailToolbarBackend, self).__init__(*args, **kwargs)

        self.outbox = load_outbox()

    def send_messages(self, messages):
        """Redirect messages to the cached outbox"""

        for message in messages:
            new_message = MailToolbarBackendEmail(message)
            self.outbox[new_message.id] = new_message

        save_outbox(self.outbox)

        return super(MailToolbarBackend, self).send_messages(messages)



from django.core.cache import caches

try:
    CACHE_SETTINGS = caches.settings
except AttributeError:  # < Django 3.2
    from django.conf import settings

    CACHE_SETTINGS = settings.CACHES

from .conf import MAIL_TOOLBAR_CACHE_KEY, MAIL_TOOLBAR_TTL

# Use local memory cache if default cache is a DummyCache
if not CACHE_SETTINGS.get("default", {}).get("BACKEND", "").endswith(".DummyCache"):
    from django.core.cache import cache
else:
    CACHE_SETTINGS["mail_panel"] = {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "mail-panel",
    }
    cache = caches.create_connection("mail_panel")


def load_outbox():
    """
    Returns a dictionary of cached mail.
    """
    return cache.get(MAIL_TOOLBAR_CACHE_KEY, {})


def save_outbox(outbox):
    """
    Saves the dictionary of cached mail and sets expiry.
    """
    cache.set(MAIL_TOOLBAR_CACHE_KEY, outbox, MAIL_TOOLBAR_TTL)


def clear_outbox():
    """
    Utility function to clear the dictionary of cached mail. Typical use case: Starting a new real-human test session.
    """
    cache.set(MAIL_TOOLBAR_CACHE_KEY, {})



from django.http import HttpResponse, JsonResponse
from django.conf import settings
from django.shortcuts import render
from django.utils.html import urlize
from django.utils.safestring import mark_safe

from .utils import load_outbox, save_outbox, clear_outbox

if hasattr(settings, "DEBUG_TOOLBAR_FILTER_URL"):
    settings.DEBUG_TOOLBAR_FILTER_URL = settings.DEBUG_TOOLBAR_FILTER_URL + ("__mail_toolbar_debug__")
else:
    settings.DEBUG_TOOLBAR_FILTER_URL = ("__mail_toolbar_debug__",)


def load_message(request, message_id):
    """
    Loads a message template into the subframe
    """
    message = None
    mail_list = load_outbox()
    message = mail_list.get(message_id, None)

    alternatives = ["message/rfc822"]
    if message:
        message.read = True

        save_outbox(mail_list)

        if message.body:
            if hasattr(message, 'content_subtype') and message.content_subtype == "html":
                alternatives.append("text/html")
            else:
                alternatives.append("text/plain")
        if hasattr(message, "alternatives"):
            for alternative in message.alternatives:
                alternatives.append(alternative[1])

    return render(request, 'mail_panel/message_overview.html', dict(
        message=message,
        alternatives=alternatives,
    ))


def display_multipart(request, message_id, multipart):
    """
    Displays the requested multipart from the email message.
    """

    mail_list = load_outbox()
    message = mail_list.get(message_id, None)
    if not message:
        return HttpResponse('Messsage has expired from cache.')

    if multipart not in ('', 'text/plain'):
        if multipart == "message/rfc822":
            return render(request, "mail_panel/raw_message.html", dict(
                body=message.message().as_string())
            )
        elif hasattr(message, "alternatives"):
            for alternative in message.alternatives:
                if alternative[1] == multipart:
                    body = mark_safe(alternative[0].replace("<a ", "<a target='_blank' "))
                    return HttpResponse(body)
        else:
            body = mark_safe(message.body.replace("<a ", "<a target='_blank' "))
            return HttpResponse(body)


    return render(request, "mail_panel/plain_text_message.html", dict(
        body=mark_safe(
            urlize(message.body).replace("<a ", "<a target='_blank' "))
    ))


def download_attachment(request, message_id, attachment_id):
    """
    Downloads an attachment from a message
    """

    mail_list = load_outbox()
    message = mail_list.get(message_id, None)
    if not message:
        return HttpResponse('Messsage has expired from cache.')

    filename, content, content_type = message.attachments[int(attachment_id)]
    response = HttpResponse(content)

    if content_type:
        response['Content-Type'] = content_type

    # Force download as attachment
    response['Content-Disposition'] = 'attachment; filename={0}'.format(
        filename
    )

    return response

def clear_message(request, message_id):
    """
    Clears a message from the cache
    """
    mail_list = load_outbox()
    try:
        del(mail_list[message_id])
    except KeyError:
        pass
    save_outbox(mail_list)

    return JsonResponse({"status": "success"})

def clear_all_messages(request):
    """
    Clears all message from the cache
    """
    clear_outbox()

    return JsonResponse({"status": "success"})



from django.urls import include, path, re_path

from . import views

mail_toolbar_patterns = [
    path("load_message/<slug:message_id>/", views.load_message, name="load_message"),
    path(
        "download_attachment/<slug:message_id>/<int:attachment_id>/",
        views.download_attachment,
        name="download_attachment",
    ),
    re_path(
        r"^display_multipart/(?P<message_id>[\w]+)/(?P<multipart>[\w/]+)$",
        views.display_multipart,
        name="display_multipart",
    ),
    path("clear_message/<slug:message_id>/", views.clear_message, name="clear_message"),
    path("clear_all_messages/", views.clear_all_messages, name="clear_all_messages"),
]

urlpatterns = [
    path("mail_toolbar/", include(mail_toolbar_patterns)),
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mail_panel']

package_data = \
{'': ['*'],
 'mail_panel': ['static/debug_toolbar/mail/*', 'templates/mail_panel/*']}

install_requires = \
['django-debug-toolbar>=1.0']

setup_kwargs = {
    'name': 'django-mail-panel',
    'version': '4.0.0',
    'description': 'A panel for django-debug-toolbar that allows for viewing of recently sent email.',
    'long_description': "\n## Django Debug Toolbar - Mail Panel\n\n[![Build Status](https://secure.travis-ci.org/scuml/django-mail-panel.png?branch=master)](http://travis-ci.org/scuml/django-mail-panel)\n\n![](https://cloud.githubusercontent.com/assets/1790447/9289964/6aa7c4ba-434e-11e5-8594-3bb3efd0cd81.png)\n\n\nTesting and debugging e-mail while developing a Django app has never been pleasant.  Sending e-mail to a file-based backend requires a user to click through obtusely-named files and does not provide a way to preview rendered HTML.  Sending e-mail to a valid mailbox incurs a delay as the message is processed though a mail server, and clutters a developer's inbox.\n\nThe mail panel attempts to address these problems by providing a way to preview emails within the browser using [django-debug-toolbar](https://github.com/jazzband/django-debug-toolbar).\n\nThis mail panel is released under the Apache license. If you like it, please consider contributing!\n\nSpecial thanks to @ShawnMilo for the code review.\n\n\nInstallation\n============\n\nTo install the mail panel, first install this package with `pip install django-mail-panel`.  Then add the `mail_panel` app after `debug_toolbar`to the `INSTALLED_APPS` setting:\n\n```python\nINSTALLED_APPS = (\n    ...\n    'debug_toolbar',\n    'mail_panel',\n)\n```\n\nand add the panel `DEBUG_TOOLBAR_PANELS`:\n\n```python\nDEBUG_TOOLBAR_PANELS = (\n    ...\n    'mail_panel.panels.MailToolbarPanel',\n)\n```\n\n\nCollect static and you'll be good to go.\n\n```bash\n./manage.py collectstatic\n```\n\n\nConfiguration\n=============\n\nAfter installation, you now need to redirect mail to the mail toolbar.  Change your email backend to the following:\n\n```python\nEMAIL_BACKEND = 'mail_panel.backend.MailToolbarBackend'\n```\n\n**Important:** This plugin uses Django's cache backend to store messages.  If you are using `DummyCache`, the mail panel will use a local memory cache, and will reset messages when the server is restarted.\n\n\n**[Optional]** \nBy default, mail toolbar stores messages for one day before removing them from cache.  You can change this with the following setting:\n\n```python\nMAIL_TOOLBAR_TTL = 86400  # 1 Day\n```\n\n**[Optional]**\nIf you use the `DEBUG_TOOLBAR_PANELS` to custom order your panels:\n\n```python\n    DEBUG_TOOLBAR_PANELS = [\n        'debug_toolbar.panels.versions.VersionsPanel',\n        'debug_toolbar.panels.timer.TimerPanel',\n        'debug_toolbar.panels.settings.SettingsPanel',\n        'debug_toolbar.panels.headers.HeadersPanel',\n        'debug_toolbar.panels.request.RequestPanel',\n        'debug_toolbar.panels.sql.SQLPanel',\n        'debug_toolbar.panels.staticfiles.StaticFilesPanel',\n        'debug_toolbar.panels.templates.TemplatesPanel',\n        'debug_toolbar.panels.cache.CachePanel',\n        'debug_toolbar.panels.signals.SignalsPanel',\n        'debug_toolbar.panels.logging.LoggingPanel',\n        'debug_toolbar.panels.redirects.RedirectsPanel',\n        'mail_panel.panels.MailToolbarPanel',\n    ]\n```\n\n\n\nTesting\n=======\n\nTo preview emails sent from your test suite, add the email backend override to your tests with the following:\n\n```python\nfrom django.test.utils import override_settings\n\n@override_settings(EMAIL_BACKEND='mail_panel.backend.MailToolbarBackend')\ndef test_send_email(self):\n    # your code here\n```\n\n\nThe backend works similarly to the standard email backend and code should not need to be reworked when using the MailToolbarBackend.\n\n\n```python\nfrom django.core import mail\n\noriginal_outbox = len(mail.outbox)\n# Send mail ...\nassert(len(mail.outbox) == original_outbox + 1)\n```\n\nShameless Plugs\n=======\nLike Django Mail Panel?  Be sure to check out and support these other tools for Mac that will improve your workflow:\n\n**[Kubermagic](https://echodot.com/kubermagic/)** - Automate, and script away tedious kubectl commands with Kubermagic; a UI for developers, QA teams, and those starting to learn the ins-and-outs of Kubernetes.     \n\n\n**[Red](https://echodot.com/red/)** - A visual and interactive Redis client, featuring live updating keys, an interactive console, pub/sub, lua script support and much more.\n",
    'author': 'Stephen Mitchell',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/scuml/django-mail-panel',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
